@@ -61,7 +61,7 @@ static const size_t numButtons = sizeof(buttonNames) / sizeof(*buttonNames);
 			[buttons setObject: button forKey: buttonNames[i]];
 		}
 		[buttons makeImmutable];
-		_buttons = [buttons retain];
+		_buttons = [buttons copy];
 
 		up = [OHGameControllerButton oh_elementWithName: @"D-Pad Up"
 							 analog: false];
@@ -85,7 +85,7 @@ static const size_t numButtons = sizeof(buttonNames) / sizeof(*buttonNames);
 
 		objc_autoreleasePoolPop(pool);
 	} @catch (id e) {
-		[self release];
+		objc_release(self);
 		@throw e;
 	}
 
@@ -94,8 +94,8 @@ static const size_t numButtons = sizeof(buttonNames) / sizeof(*buttonNames);
 
 - (void)dealloc
 {
-	[_buttons release];
-	[_directionalPads release];
+	objc_release(_buttons);
+	objc_release(_directionalPads);
 
 	[super dealloc];
 }
